//
//  ContentMainTabHomeViewModel.swift
//  SodaLive
//
//  Created by klaus on 2/20/25.
//

import Foundation
import Combine

final class ContentMainTabHomeViewModel: ObservableObject {
    
    private let repository = ContentMainTabHomeRepository()
    private let contentRepository = ContentRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    @Published var isLoading = false
 
    @Published var noticeItem: NoticeItem? = nil
    @Published var bannerList = [GetAudioContentBannerResponse]()
    @Published var rankCreatorResponse: GetExplorerSectionResponse? = nil
    @Published var rankSeriesList = [SeriesListItem]()
    @Published var rankSortTypeList: [String] = []
    @Published var rankContentList: [GetAudioContentRankingItem] = []
    @Published var eventBannerList: [EventItem] = []
    @Published var contentRankCreatorList: [ContentCreatorResponse] = []
    @Published var salesCountRankContentList: [GetAudioContentRankingItem] = []
    
    func fetchData() {
        isLoading = true
        
        repository.getContentMainHome()
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetContentMainTabHomeResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.noticeItem = data.latestNotice
                        self.bannerList = data.bannerList
                        self.rankCreatorResponse = data.rankCreatorList
                        self.rankSeriesList = data.rankSeriesList
                        self.rankSortTypeList = data.rankSortTypeList
                        self.rankContentList = data.rankContentList
                        self.eventBannerList = data.eventBannerList.eventList
                        self.contentRankCreatorList = data.contentRankCreatorList
                        self.salesCountRankContentList = data.salesCountRankContentList
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func getContentRanking(sort: String = "매출") {
        isLoading = true
        contentRepository.getContentRanking(page: 1, size: 12, sortType: sort)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetAudioContentRanking>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.rankContentList = data.items
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func getPopularContentByCreator(creatorId: Int) {
        isLoading = true
        repository.getPopularContentByCreator(creatorId: creatorId)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<[GetAudioContentRankingItem]>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.salesCountRankContentList = data
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
}
