//
//  TextMessageViewModel.swift
//  SodaLive
//
//  Created by klaus on 2023/08/10.
//

import Foundation
import Combine

final class TextMessageViewModel: ObservableObject {
    private let repository = MessageRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var items = [TextMessageItem]()
    
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    @Published var isLoading = false
    
    @Published var currentFilter: MessageFilterTab = .receive {
        willSet(newVal) {
            page = 1
            isLast = false
            
            switch newVal {
            case .receive:
                getReceivedTextMessage()
                
            case .sent:
                getSentTextMessage()
                
            case .keep:
                getKeepTextMessage()
            }
        }
    }
    
    @Published var message: String = ""
    @Published var recipientNickname: String = ""
    @Published var recipientId = 0
    
    @Published var sendText = "메시지 보내기"
    
    let placeholder = "내용을 입력해 주세요."
    
    var isLast = false
    var page = 1
    private let size = 10
    
    func write() {
        let textMessage = message.trimmingCharacters(in: .whitespacesAndNewlines) != placeholder ? message : ""
        if recipientId <= 0 {
            errorMessage = "받는 사람을 선택해 주세요."
            isShowPopup = true
            return
        }
        
        if textMessage.count < 10 {
            errorMessage = "10글자 이상 입력해 주세요."
            isShowPopup = true
            return
        }
        
        if !isLoading {
            isLoading = true
            let request = SendTextMessageRequest(recipientId: recipientId, textMessage: textMessage)
            repository.sendTextMessage(request: request)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { response in
                    self.isLoading = false
                    let responseData = response.data
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                        
                        if decoded.success {
                            self.errorMessage = "메시지 전송이 완료되었습니다."
                            self.isShowPopup = true
                            DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
                                AppState.shared.back()
                            }
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                }
                .store(in: &subscription)
        }
    }
    
    func getReceivedTextMessage() {
        if page == 1 {
            items.removeAll()
        }
        
        isLoading = true
        
        repository
            .getReceivedTextMessage(page: page, size: size)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetTextMessageResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        if data.items.count <= 0 {
                            self.isLast = true
                        } else {
                            self.items.append(contentsOf: data.items)
                            self.page += 1
                        }
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
    
    func getSentTextMessage() {
        if page == 1 {
            items.removeAll()
        }
        
        isLoading = true
        
        repository
            .getSentTextMessage(page: page, size: size)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetTextMessageResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        if data.items.count <= 0 {
                            self.isLast = true
                        } else {
                            self.items.append(contentsOf: data.items)
                            self.page += 1
                        }
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
    
    func getKeepTextMessage() {
        if page == 1 {
            items.removeAll()
        }
        
        isLoading = true
        
        repository
            .getKeepTextMessage(page: page, size: size)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetTextMessageResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        if data.items.count <= 0 {
                            self.isLast = true
                        } else {
                            self.items.append(contentsOf: data.items)
                            self.page += 1
                        }
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
}
