//
//  NicknameUpdateViewModel.swift
//  SodaLive
//
//  Created by klaus on 2023/08/19.
//

import Foundation
import Combine

import Moya

final class NicknameUpdateViewModel: ObservableObject {
    private let repository = UserRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var isLoading = false
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    
    @Published var nickname = "" {
        didSet {
            if nickname.count > 12 {
                nickname = String(nickname.prefix(12))
            }
            
            isCheckedNickname = false
        }
    }
    @Published var price = 0
    
    var isCheckedNickname = false
    
    func getChangeNicknamePrice() {
        isLoading = true
        
        repository.getChangeNicknamePrice()
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetChangeNicknamePriceResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.price = data.price
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    ERROR_LOG(error.localizedDescription)
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
    
    func checkNickname() {
        if !nickname.trimmingCharacters(in: .whitespaces).isEmpty {
            isLoading = true
            
            repository.checkNickname(nickname: nickname)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { [unowned self] response in
                    self.isLoading = false
                    let responseData = response.data
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                        
                        if decoded.success {
                            self.isCheckedNickname = true
                            self.errorMessage = "사용가능한 닉네임 입니다."
                            self.isShowPopup = true
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                }
                .store(in: &subscription)
        } else {
            self.errorMessage = "닉네임을 입력하세요."
            self.isShowPopup = true
        }
    }
    
    func changeNickname() {
        if isCheckedNickname {
            isLoading = true
            
            let request = ProfileUpdateRequest(email: UserDefaults.string(forKey: .email), nickname: nickname)
            repository.changeNickname(request: request)
                .sink { result in
                    switch result {
                    case .finished:
                        DEBUG_LOG("finish")
                    case .failure(let error):
                        ERROR_LOG(error.localizedDescription)
                    }
                } receiveValue: { [unowned self] response in
                    self.isLoading = false
                    let responseData = response.data
                    
                    do {
                        let jsonDecoder = JSONDecoder()
                        let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                        
                        if decoded.success {
                            UserDefaults.set(nickname, forKey: .nickname)
                            self.errorMessage = "닉네임이 변경되었습니다."
                            self.isShowPopup = true
                            
                            DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
                                AppState.shared.back()
                            }
                        } else {
                            if let message = decoded.message {
                                self.errorMessage = message
                            } else {
                                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            }
                            
                            self.isShowPopup = true
                        }
                    } catch {
                        self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        self.isShowPopup = true
                    }
                }
                .store(in: &subscription)
            
        } else {
            self.errorMessage = "닉네임 중복체크를 해주세요."
            self.isShowPopup = true
        }
    }
}
